"""Utility programs for examining output from Tlusty

At the moment, it contains the following routines:

1) pconv - for ploting the convergence log contasined in the output
file from Tlusty (fort.9);

2) pmodel - extracting values of column mass and a selected state parameter
from the Tlusty output file fort.7m (coindensed model) or fort.12 
(b-factors for NLTE models);

3) pmodels - similar to pmodel, but with an option to plot selected state 
parameter for one or more Tlusty models, and a possibility of
plotting differences of between individual models

4) pmods - a plot of a range pf state parameters for one or more Tlusty models

Examples:
--------

To plot a convergence log of the last computed model:

>>> import matplotlib.pyplot as plt
>>> import tlusty as tl
>>> plt.ion()
>>> tl.pconv()

a convergence log for a previously computed model, say, hhe35lt:

>>> tl.pconv('hhe35lt')

To extract log(column mass) and temperature of the same model
 
>>> (m35,t35) = tl.pmodel('hhe35lt')

which can then be plotted or used for some other purpose.
 
To plot the temperature as a function of log(m) for the last computed
model, do

>>> tl.pmodels()

and to plot the temperature for three models, hhe35lt','hhe35nc, and
hhe35nl, and then differences from the first model, do

>>> tl.pmodels(['hhe35lt','hhe35nc','hhe35nl'])
>>> tl.pmodels(['hhe35lt','hhe35nc','hhe35nl'],diff=True)

To plot level populations for the first five levels of hydrogen for the
model hhe35nl, do

>>> tl.pmods('hhe35nl',[3,4,5,6,7])


"""


import matplotlib.pyplot as plt
import numpy as np
import sys

###################################################

def pconv(relcfile='fort'):

    """
    Plots a convergence log - output file unit.9 from Tlusty.
    The routine is analogous to the old IDL routine PCONV.PRO
    (or RTELC7.PRO). It produces a 2 x 3 array of subplots:
    the first row plots the relative chasnges in temperature (or, if 
    the temperature was not changed in the model cnstruction, then the
    maximum relative change of the level populations);
    the seconw show tha maximum relative change of all state parameters. 
    The left column shows the actual values of the relative changes as
    a function of depth index; each line represents one iteration;
    the middle column shows the logarithms of the absolute values of the
    same quantities, again as a function of depth index, and
    the right column shows the log of maximum value of all changes in 
    all depths as a function of the iteration number.

    The right column of plots is most indicative for the overall convergence
    of the model - the maximum changes should gradually decrease with an 
    iteration step.

    The title of the plot also shows elapsed time (in sec) for running the
    Tlusty model, if a timing log (output file unit 69) is available.

    Parameters:
    -----------
    relcfile: str, optional
        core name for the model atmosphere file, the code adds '.9'
        default: 'fort', i.e. the code takes the file name as 'fort.9'

    Returns: none

"""   

    relcfile = relcfile+'.9'

    try:
        f = open(relcfile)
        lines = f.readlines()
    except:
        print('Error: Could not open %s' % relcfile)
        return()

    idep,itnu,delt,delp,delm = [],[],[],[],[]
    i = 3
    while i < len(lines):
        xlin = lines[i]
        xlin = xlin.replace('D','E',5)
        xlin = xlin.strip()
        val  = xlin.split()
        itnu.append(float(val[0]))
        idep.append(float(val[1]))
        delt.append(float(val[2]))
        delp.append(float(val[4]))
        delm.append(float(val[6]))
        i += 1

    niter = int(itnu[-1])
    nd = int(idep[0])

    xtit1 = 'depth index'
    xtit2 = 'iteration'
    ytit1 = 'rel.change'
    ytit2 = 'log(rel.change)'
    ftit1 = 'temperature'
    ftit2 = 'max.population'
    ftit3 = 'max in state vector'

    if max(delt) == 0:
        delt=delp
        ftit1 = ftit2
        
    print(delt[0],delt[-1])
   
    fig, axes = plt.subplots(2,3)

    axes[0, 0].set_xlabel(xtit1)
    axes[0, 0].set_ylabel(ytit1)
    axes[0, 0].set_title(ftit1)

    axes[0, 1].set_xlabel(xtit1)
    axes[0, 1].set_ylabel(ytit2)
#   axes[0, 1].set_title(ftit1)

    axes[1, 0].set_xlabel(xtit1)
    axes[1, 0].set_ylabel(ytit1)
    axes[1, 0].set_title(ftit3)

    axes[1, 1].set_xlabel(xtit1)
    axes[1, 1].set_ylabel(ytit2)
    axes[1, 1].set_title(ftit3)

    i=0
    tmaxt, tmaxm, tmaxi = [], [], []
    while i < niter:
        i0, i1 = i*nd, (i+1)*nd-1
        axes[0, 0].plot(idep[i0:i1],delt[i0:i1])
        axes[0, 1].plot(idep[i0:i1],np.log10(np.abs(delt[i0:i1])))
        axes[1, 0].plot(idep[i0:i1],delm[i0:i1])
        axes[1, 1].plot(idep[i0:i1],np.log10(np.abs(delm[i0:i1])))
        tmaxt.append(max(np.log10(np.abs(delt[i0:i1]))))
        tmaxm.append(max(np.log10(np.abs(delm[i0:i1]))))
        i+=1
        tmaxi.append(i)

    axes[0, 2].set_xlabel(xtit2)
    axes[0, 2].set_ylabel('max(rel.changei)')
    axes[0, 2].set_title(ftit1)
#   axes[0, 2].plot(tmaxi,tmaxt,'ko',ls=':',ms=10)
    axes[0, 2].plot(tmaxi,tmaxt,'ko',ls='--')

    axes[1, 2].set_xlabel(xtit2)
    axes[1, 2].set_ylabel('max(rel.change')
    axes[1, 2].set_title(ftit3)
#   axes[1, 2].plot(tmaxi,tmaxm,'ko',ls=':',ms=10)
    axes[1, 2].plot(tmaxi,tmaxm,'ko',ls='--')

#   overall title

#   contains the cores name of the model, and also shows  the total 
#   execution time used by Tlusty to generate the model form file 
#   [core name].69 (if it exists)

    timfile = relcfile[0:-2]+'.69'
    try:
        ft = open(timfile)
        linti = ft.readlines()
        titlt = 'time = '+ linti[-1].split()[2] +' s'
    except:
        print('Error: Could not open %s' % relcfile)
        titlt=' '
        

    titl=relcfile + ':  ' + titlt

    fig.suptitle(titl)
    fig.tight_layout()
    plt.show()

# Next lines permit one to run the routine from the command line
#if __name__ == "__main__":
#    import sys
#    if len(sys.argv)==4:
#        doit(sys.argv[1],sys.argv[2],sys.argv[3])
#    else:
#        print (__doc__)

#   return()


####################################################


def pmodel(modfile='fort', ind=0):

    """ 
    Extracts column mass and one state parameter from the Tlusty
    condensed model atmopshere file, unit 7. It can also extracts
    b-factors from an analogous output file, unit 12.
    The routine is analogous to an old IDL routine PMODEL.PRO (or
    MODELS11.PRO).

    Paramaters:
    -----------
    modfile: str, optional
        core name for the model atmosphere file. If the name does not
        contain a period ("."), the code adds ".7"
        default: 'fort', i.e. the code takes the file name as 'fort.7'
    ind: integer, optional - the index of the state parameter
        default 0 , i.e. the temperature

    Indices of the state parameters are as follows:
    0 - temperature (K)
    1 - electron density (cm**-3)
    2 - mass density (g cm **-3)
    3 - the total particle numer density (only for models with moleculaes)
    following - level populations

    Returns: none

"""


#   import matplotlib.pyplot as plt
#   import numpy as np
#   import sys

    if modfile.count('.') == 0:
       modfile=modfile+'.7'

    try:
        f=open(modfile)
        lines=f.readlines()
    except:
        print('Error: Could not open %s' % modfile)
        return() 

#   read number of depths of imput parameters

    x=lines[0]
    nd=int(x.split()[0])
    npa=abs(int(x.split()[1]))

# read depths

    ndrow=6
    if modfile.count('.12') > 0:
        ndrow=8

    nr=nd//ndrow
    if nd % ndrow != 0:
        nr=nr+1

#   print(ndrow,nr)
    dm=[]
    i=0
    while i < nr:
        x=lines[i+1]
        x=x.replace('D','E',6)
        val=x.split()
        j=0
        while j< ndrow:
            k=ndrow*i+j
            if k < nd:
                dm.append(float(val[j]))
            j+=1
        i+=1

# read state parameters

    nrx = npa//5
    if npa % 5 != 0:
        nrx=nrx+1

    print(npa,nrx)
    par=[]
    i=nr+1
    id=0
    while id < nd:
        xpar=[]
        j=0
        while j < nrx:
            x=lines[i]
            x=x.replace('D','E',6)
            val=x.split()
            k=0
            xpar0=xpar
            while k < 5:
                if 5*j+k < npa:
                    xpar.append(float(val[k]))
                k+=1
            j+=1
            i+=1
        par.append(xpar[ind])
        id+=1

#   plot(s)
   
    dm=np.log10(dm)
    par=np.array(par)
    if ind > 0: par=np.log10(par)

    return(dm,par)

#   fig, ax = plt.subplots()
#   ax.set_xlabel('log mass')
#   ax.set_title(modfile)
#   if ind == 0:
#      ax.set_ylabel=('temperature')
#      ax.plot(dm,par)
#   else:
#      ax.set_ylabel('log n')
#      ax.plot(dm,np.log10(par))

#   plt.show()


####################################################


def pmodels(modfiles=['fort'], ind=0, diff=False):

    """ 
    Similar to pmodel; represents its extended version.
    It plots the values of one state parameter from the Tlusty
    condensed model atmosphere file, unit 7 (or unit 12), for
    oner or more models. It can also plot differences between
    models. 

    Paramaters:
    -----------
    modfiles: array or list of strings, optional
        Each string represents a core name for the model atmosphere 
        file. If the name does not contain a period ("."), the code adds ".7"
        default: 'fort', i.e. the code takes the file name as 'fort.7'
    ind: integer, optional - the index of the state parameter
        default: 0 , i.e temperature
    diff: Bolean, optional
        if True,the plot contains differences of the state parameter
        specified by the parameter ind from the first model of the list
        specified by the parameter modfiles; otherwise the actual values
        of the state parameter are plotted;
        default: False; i.e. the actual values are plotted

    Indices of the state parameters are as follows:
    0 - temperature (K)
    1 - electron density (cm**-3)
    2 - mass density (g cm **-3)
    3 - the total pareticle numer density (only for models with moleculaes)
    following - level populations

    Returns: none

"""


#   import matplotlib.pyplot as plt
#   import numpy as np
#   import sys

#   loop over all models to be plotted

    imod = 0
    for modfile in modfiles:

        if modfile.count('.') == 0:
           modfile=modfile+'.7'

        try:
            f=open(modfile)
            lines=f.readlines()
        except:
            print('Error: Could not open %s' % modfile)
            return()


# read number of depths of imput parameters

        x=lines[0]
        nd=int(x.split()[0])
        npa=abs(int(x.split()[1]))

# read depths

        ndrow=6
        if modfile.count('.12') > 0:
            ndrow=8

        nr=nd//ndrow
        if nd % ndrow != 0:
            nr=nr+1

        dm=[]
        i=0
        while i < nr:
            x=lines[i+1]
            x=x.replace('D','E',6)
            val=x.split()
            j=0
            while j< ndrow:
                k=ndrow*i+j
                if k < nd:
                    dm.append(float(val[j]))
                j+=1
            i+=1

# read state parameters

        nrx = npa//5
        if npa % 5 != 0:
            nrx=nrx+1

#       print(npa,nrx)
        par=[]
        i=nr+1
        id=0
        while id < nd:
            xpar=[]
            j=0
            while j < nrx:
                x=lines[i]
                x=x.replace('D','E',6)
                val=x.split()
                k=0
                xpar0=xpar
                while k < 5:
                    if 5*j+k < npa:
                        xpar.append(float(val[k]))
                    k+=1
                j+=1
                i+=1
            par.append(xpar[ind])
            id+=1

#  plot(s)
   
        dm=np.log10(dm)
        if ind != 0:
           par=np.log10(par)

        if imod == 0:
            fig, ax = plt.subplots()
            ax.set_xlabel('log mass')
            titl=modfile[:-2]
#           ax.set_title(modfile)
            dm0=dm
            par0=par
            if diff == True:
               par=np.array(par)-np.array(par0)
        else:
            titl = titl+', '+modfile[:-2]
            if diff == True:
                pari=np.interp(dm0,dm,par)
                par=pari-par0

        if ind == 0:
           ax.set_ylabel=('temperature')
        else:
           ax.set_ylabel('log n')

        if diff == False:
           ax.plot(dm,par)
        else:
           ax.plot(dm0,par)

        imod+=1

    ax.set_title(titl)
    plt.show()


#####################################################################


def pmods(modfiles, ind):

    """ 
    Similar to pmodels; represents its extended version.
    It plots the values of one or more state parameters from the Tlusty
    condensed model atmosphere file, unit 7 (or unit 12), for
    one or more models.

    Paramaters:
    -----------
    modfiles: array or list of strings
        Each string represents a core name for the model atmosphere
        file. If the name does not contain a period ("."), the code adds ".7"
    ind: array or list of integers - the indices of the state parameters
        to be plotted

    Indices of the state parameters are as follows:
    0 - temperature (K)
    1 - electron density (cm**-3)
    2 - mass density (g cm **-3)
    3 - the total pareticle numer density (only for models with moleculaes)
    following - level populations

    Returns: none

"""


#   import matplotlib.pyplot as plt
#   import numpy as np
#   import sys

#   loop over all models to be plotted

    imod = 0
    for modfile in modfiles:

        if modfile.count('.') == 0:
           modfile=modfile+'.7'

        try:
            f=open(modfile)
            lines=f.readlines()
        except:
            print('Error: Could not open %s' % modfile)
            return()

#   read number of depths of imput parameters

        x=lines[0]
        nd=int(x.split()[0])
        npa=abs(int(x.split()[1]))

# read depths

        ndrow=6
        if modfile.count('.12') > 0:
            ndrow=8

        nr=nd//ndrow
        if nd % ndrow != 0:
            nr=nr+1

        dm=[]
        i=0
        while i < nr:
            x=lines[i+1]
            x=x.replace('D','E',6)
            val=x.split()
            j=0
            while j< ndrow:
                k=ndrow*i+j
                if k < nd:
                    dm.append(float(val[j]))
                j+=1
            i+=1

# read state parameters

        nrx = npa//5
        totpar=[]
        if npa % 5 != 0:
            nrx=nrx+1

#       print(npa,nrx)
        par=[]
        i=nr+1
        id=0
        while id < nd:
            xpar=[]
            j=0
            while j < nrx:
                x=lines[i]
#               print(i,x)
                x=x.replace('D','E',6)
                val=x.split()
                k=0
                while k < 5:
                    if 5*j+k < npa:
                        xpar.append(float(val[k]))
                    k+=1
                j+=1
                i+=1
            totpar.append(xpar)
            id+=1
        pararr=np.array(np.log10(totpar))

#   plot(s)

        dm=np.log10(dm)

        if imod == 0:
            fig, ax = plt.subplots()
            ax.set_xlabel('log mass')
            ax.set_ylabel('log n')
            ax.set_title(modfile)

        for indx in ind:
            ax.plot(dm,pararr[:,indx])
#       if ind == 0:
#          ax.set_ylabel=('temperature')
#          ax.plot(dm,par)
#       else:
#          ax.set_ylabel('log n')
#          ax.plot(dm,np.log10(par))

        imod+=1

    plt.show()



def pflux(modfiles=['fort'],rel=False,ilogx=False):

    """
    Plots the emergent flux, or a relative difference of fluxes,
    from the output file, unit 14, from Tlusty; or a synthetic
    spectrum computed by Synspec.

    Paramaters:
    -----------
    modfiles: array or list of strings, optional
        Each string represents a core name for the model atmosphere
        file. If the name does not contain a period ("."), the code adds ".14"
        default: 'fort', i.e. the code takes the file name as 'fort.14
    rel: Bolean, optional
        if False, it plots the actual values of the fluxe;
        if True, it plots the fluxes relative to the first model in %, i.e.
        rel(i) = (flux(i)-flux(0))/flux(0)*100.
    ilogx: Bolean, optional
        if True, sets a logarithmic scale on x-axis

    Returns: none

    """



    imod = 0
    for modfile in modfiles:

        if modfile.count('.') == 0:
            modfile=modfile+'.14'

        wl,fl = np.loadtxt(modfile,unpack=True)

        if imod == 0:
            fig, ax = plt.subplots()
            ax.set_xlabel('wavelength')
            titl=modfile[:-3]

            wl0 = wl
            fl0 = fl
            titl = modfile[:-3]
            if rel == True:
                fl = fl - fl0
        else:
            titl = titl + ', '+modfile[:-3]
            if rel == True:
               fli = np.interp(wl0,wl,fl)
               fl  = (fli-fl0)/fl0*100.

        if rel == False:
            if ilogx == False:
                ax.plot(wl,fl)
            else:
                ax.semilogx(wl,fl)
        else:
            if ilogx == False:
                ax.plot(wl0,fl)
            else:
                ax.semilogx(wl0,fl)

        imod +=1

    ax.set_title(titl)
    plt.show()

if __name__ == "__main__":

    file = sys.argv[1]
    pconv(file)
